{% ckan_extends %}

{% block action_manage %}
{{ super() }}
{% if res.url and h.is_url(res.url) and res.format=='SPARQL' %}
<li>{% link_for _('Query'), named_route='fuseki.query', id=pkg.name, class_='btn btn-secondary', icon='play' %}</li>
{% endif %}
{% endblock action_manage %}